//
//  ServiceOperation.m
//  AppPiza
//
//  Copyright (c) 2014 CJSC "Computing Forces"
//

#import "ServiceOperation.h"

#import "URLConnectionContainer.h"



@interface ServiceOperation () <URLConnectionContainerDelegate>

@property (nonatomic, readwrite, assign) ServiceOperationState state;

@end




@implementation ServiceOperation {
    URLConnectionContainer *_connection;
}

- (instancetype)initWithDataSource:(id<ServiceOperationDataSource>)dataSource delegate:(id<ServiceOperationDelegate>)delegate {
    self = [super init];
    if (self) {
        self.dataSource = dataSource;
        self.delegate = delegate;
    }
    return self;
}



- (void)dealloc {
    [_dataSource release];
    
    [super dealloc];
}



- (void)prepareForQueue {
    self.state = ServiceOperationStateQueued;
}



#pragma mark -
#pragma mark Base background methods
- (void)main {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    
    @try {
        self.state = ServiceOperationStateRunning;
        
        // Prepare HTTP request
        NSURL *url = [self.dataSource respondsToSelector:@selector(endpointURLForOperation:)] ? [self.dataSource endpointURLForOperation:self] : nil;
        NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
        
        if ([self.dataSource respondsToSelector:@selector(httpMethodForOperation:)]) {
            NSString *httpMethod = [self.dataSource httpMethodForOperation:self];
            if (self.isCancelled) return;
            
            [request setHTTPMethod:httpMethod];
        }
        
        if ([self.dataSource respondsToSelector:@selector(httpBodyDataForOperation:)]) {
            NSData *httpBody = [self.dataSource httpBodyDataForOperation:self];
            if (self.isCancelled) return;
            
            [request setHTTPBody:httpBody];
        }
        
        if ([self.dataSource respondsToSelector:@selector(httpHeadersForOperation:)]) {
            NSDictionary *httpHeaders = [self.dataSource httpHeadersForOperation:self];
            if (self.isCancelled) return;
            
            [request setAllHTTPHeaderFields:httpHeaders];
        }
        
        NSStringEncoding responseStringEncoding = 0;
        if ([self.dataSource respondsToSelector:@selector(responseStringEncodingForOperation:)]) {
            responseStringEncoding = [self.dataSource responseStringEncodingForOperation:self];
            if (self.isCancelled) return;
        }
        
        // Log request
#ifdef DEBUG
        NSLog(@"REQUEST: %@ %@", request.HTTPMethod, request.URL);
        
        if ([request.allHTTPHeaderFields count]) {
            NSLog(@"HEADERS:");
            for (NSString *key in request.allHTTPHeaderFields)
                NSLog(@"%@ = %@", key, request.allHTTPHeaderFields[key]);
        }
        else {
            NSLog(@"NO HEADERS");
        }
        
        if (request.HTTPBody.length) {
            NSString *bodyString = [[NSString alloc] initWithData:request.HTTPBody
                                                         encoding:NSWindowsCP1251StringEncoding];
            NSLog(@"BODY: %@", bodyString);
            [bodyString release];
        }
        else {
            NSLog(@"NO BODY");
        }
#endif
        
        // Start HTTP request
        _connection = [[URLConnectionContainer alloc] initWithURLRequest:request];
        _connection.delegate = self;
        
        if (responseStringEncoding)
            _connection.responseStringEncoding = responseStringEncoding;
        
        [_connection startAsync:NO];
//        [NSThread sleepForTimeInterval:2.0];
        
        // Log response
#ifdef DEBUG
        NSLog(@"RESPONSE STATUS CODE: %d", (int)((NSHTTPURLResponse *)_connection.urlResponse).statusCode);
        
        if (_connection.responseData.length) {
            NSLog(@"RESPONSE DATA: %@", _connection.responseString);
        }
        else {
            NSLog(@"NO RESPONSE DATA");
        }
#endif
        
        // Call delegate
        if (_connection.error) {
            self.state = ServiceOperationStateFailed;
            [self callDelegateFailedWithError:_connection.error];
        }
        else {
            self.state = ServiceOperationStateFinished;
            [self callDelegateFinishedWithData:_connection.responseData statusCode:((NSHTTPURLResponse *)_connection.urlResponse).statusCode responseStringEncoding:responseStringEncoding];
        }
        
        // Destroy HTTP request
        [_connection release];
        _connection = nil;
    }
    @finally {
        [pool release];
    }
}



- (void)cancel {
    [super cancel];
    [_connection cancel];
    
    self.state = ServiceOperationStateCancelled;
}



- (void)cancelDueToFailure {
    [super cancel];
    [_connection cancel];
    
    self.state = ServiceOperationStateFailed;
}



- (BOOL)isFinished {
    return (self.state == ServiceOperationStateFinished || self.state == ServiceOperationStateCancelled || self.state == ServiceOperationStateFailed);
}



#pragma mark -
#pragma mark Connection container delegate
- (BOOL)connectionContainer:(URLConnectionContainer *)connectionContainer canAuthenticateAgainstProtectionSpace:(NSURLProtectionSpace *)protectionSpace {
    return YES;
}



#pragma mark -
#pragma mark Delegate callers
- (void)callDelegateStateChangedFrom:(ServiceOperationState)oldState toState:(ServiceOperationState)newState {
    if (![self.delegate respondsToSelector:@selector(serviceOperation:didChangeStateFrom:toState:)])
        return;
    
    if ([NSThread currentThread] == [NSThread mainThread]) {
        [self.delegate serviceOperation:self didChangeStateFrom:oldState toState:newState];
    }
    else {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate serviceOperation:self didChangeStateFrom:oldState toState:newState];
        });
    }
}



- (void)callDelegateFinishedWithData:(NSData *)responseData statusCode:(NSUInteger)statusCode responseStringEncoding:(NSStringEncoding)responseStringEncoding {
    if (![self.delegate respondsToSelector:@selector(serviceOperation:didFinishWithResponseData:statusCode:responseStringEncoding:)])
        return;
    
    if ([NSThread currentThread] == [NSThread mainThread]) {
        [self.delegate serviceOperation:self didFinishWithResponseData:responseData statusCode:statusCode responseStringEncoding:responseStringEncoding];
    }
    else {
        dispatch_sync(dispatch_get_main_queue(), ^{
            [self.delegate serviceOperation:self didFinishWithResponseData:responseData statusCode:statusCode responseStringEncoding:responseStringEncoding];
        });
    }
}



- (void)callDelegateFailedWithError:(NSError *)error {
    if (![self.delegate respondsToSelector:@selector(serviceOperation:didFailWithError:)])
        return;
    
    if ([NSThread currentThread] == [NSThread mainThread]) {
        [self.delegate serviceOperation:self didFailWithError:error];
    }
    else {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate serviceOperation:self didFailWithError:error];
        });
    }
}



#pragma mark -
#pragma mark Properties
- (void)setState:(ServiceOperationState)state {
    ServiceOperationState oldSTate = _state;
    _state = state;
    
    [self callDelegateStateChangedFrom:oldSTate toState:state];
}

@end
