//
//  NSURLConnectionContainer.m
//
//  Copyright (c) 2014 CJSC "Computing Forces"
//

#import "URLConnectionContainer.h"
#import <CoreFoundation/CoreFoundation.h>

#define kDelegateMethodsMode @"URLCCDelegateMethodsMode"



@interface URLConnectionContainer ()

@property (nonatomic, readwrite, retain) NSURLRequest *request;
@property (nonatomic, readwrite, retain) NSURLConnection *connection;

@property (nonatomic, readwrite, retain) NSMutableData *responseData;
@property (nonatomic, readwrite, retain) NSString *responseString;
@property (nonatomic, readwrite, retain) NSArray *responseCookies;
@property (nonatomic, readwrite, retain) NSURLResponse *urlResponse;
@property (nonatomic, readwrite, retain) NSError *error;

@end



@implementation URLConnectionContainer

static NSInteger _loadingsCount = 0;



- (id)initWithURLRequest:(NSURLRequest *)request {
    self = [super init];
    if (self) {
        self.request = request;
        self.connection = [[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:NO] autorelease];
        self.responseData = [NSMutableData data];
    }
    return self;
}



+ (id)connectionContainerWithURLRequest:(NSURLRequest *)request {
    return [[[[self class] alloc] initWithURLRequest:request] autorelease];
}



- (void)dealloc {
    [_request release];
    
    [_connection cancel];
    [_connection release];
    
    [_username release];
    [_password release];
    
    [_responseData release];
    [_responseString release];
    [_responseCookies release];
    [_urlResponse release];
    [_error release];
    
    [super dealloc];
}




#pragma mark -
#pragma mark Starting request
- (void)startAsync:(BOOL)async {
    if (_inProgress) return;
    
    _inProgress = YES;
    
    [self.responseData setLength:0];
    self.responseString = nil;
    self.responseCookies = nil;
    self.urlResponse = nil;
    self.error = nil;
    
    _bytesWritten = 0;
    _bytesExpectedToWrite = 0;
    _bytesRead = 0;
    _bytesExpectedToRead = 0;
    
    [self updateLoadingsCountWithNumber:1];
    
    if (async) {
        [_connection start];
        
    } else {
        // Starting request
        [_connection scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
        [_connection scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:kDelegateMethodsMode];
        [_connection start];

        do {
            [[NSRunLoop currentRunLoop] runMode:kDelegateMethodsMode beforeDate:[NSDate distantFuture]];
        } while (_inProgress);
    }
}



- (void)updateLoadingsCountWithNumber:(NSInteger)number {
    _loadingsCount += number;
    if (_loadingsCount > 0) [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    else if (_loadingsCount < 1) [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}



- (void)cancel {
    if (!_inProgress) return;
    
    [_connection cancel];
    
    [self updateLoadingsCountWithNumber:-1];
    _inProgress = NO;
}



#pragma mark -
#pragma mark Connection delegate
- (BOOL)connection:(NSURLConnection *)connection canAuthenticateAgainstProtectionSpace:(NSURLProtectionSpace *)protectionSpace {
    if ([_delegate respondsToSelector:@selector(connectionContainer:canAuthenticateAgainstProtectionSpace:)])
        return [_delegate connectionContainer:self canAuthenticateAgainstProtectionSpace:protectionSpace];
    
    if ([protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodClientCertificate] ||
	    [protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
		return NO;
        
    } else {
		return YES;
	}
}



- (void)connection:(NSURLConnection *)connection didCancelAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
    if ([_delegate respondsToSelector:@selector(connectionContainer:didCancelAuthenticationChallenge:)])
        [_delegate connectionContainer:self didCancelAuthenticationChallenge:challenge];
}



- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    _inProgress = NO;
    self.error = error;
    [self updateLoadingsCountWithNumber:-1];
    
    if ([_delegate respondsToSelector:@selector(connectionContainer:didFailWithError:)])
        [_delegate connectionContainer:self didFailWithError:error];
}



- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
    if ([_delegate respondsToSelector:@selector(connectionContainer:didReceiveAuthenticationChallenge:)]) {
        [_delegate connectionContainer:self didReceiveAuthenticationChallenge:challenge];
        return;
    }
    
    if ([challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
        [challenge.sender useCredential:[NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust] forAuthenticationChallenge:challenge];
//        [challenge.sender continueWithoutCredentialForAuthenticationChallenge:challenge];
        
    } else {
        if(_username && _password && [challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodHTTPBasic]) {
            if (!challenge.previousFailureCount) {
                NSURLCredential *creden = [[NSURLCredential alloc] initWithUser:_username
                                                                       password:_password
                                                                    persistence:NSURLCredentialPersistenceForSession];
                
                [[challenge sender] useCredential:creden forAuthenticationChallenge:challenge];
                [creden release];
                
            } else {
                _authorizationFailed = YES;
                [_connection cancel];
                _inProgress = NO;
                [self updateLoadingsCountWithNumber:-1];
            }
            
        } else {
            [[challenge sender] cancelAuthenticationChallenge:challenge];
        }
    }
}



- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    self.urlResponse = response;
    _bytesExpectedToRead = response.expectedContentLength;
    
    if ([_delegate respondsToSelector:@selector(connectionContainer:didReceiveResponse:)])
        [_delegate connectionContainer:self didReceiveResponse:response];
}



- (NSInputStream *)connection:(NSURLConnection *)connection needNewBodyStream:(NSURLRequest *)request {
    if ([_delegate respondsToSelector:@selector(connectionContainer:needNewBodyStream:)])
        return [_delegate connectionContainer:self needNewBodyStream:request];
    
    return nil;
}



- (NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse {
    if ([_delegate respondsToSelector:@selector(connectionContainer:willCacheResponse:)])
        return [_delegate connectionContainer:self willCacheResponse:cachedResponse];
    
    return nil;
}



- (NSURLRequest *)connection:(NSURLConnection *)inConnection willSendRequest:(NSURLRequest *)inRequest redirectResponse:(NSURLResponse *)inRedirectResponse {
    if ([_delegate respondsToSelector:@selector(connectionContainer:willSendRequest:redirectResponse:)])
        return [_delegate connectionContainer:self willSendRequest:inRequest redirectResponse:inRedirectResponse];
    
    return inRequest;
}



- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    _inProgress = NO;
    [self updateLoadingsCountWithNumber:-1];
    
    if ([_delegate respondsToSelector:@selector(connectionContainerDidFinishLoading:)])
        [_delegate connectionContainerDidFinishLoading:self];
}



- (BOOL)connectionShouldUseCredentialStorage:(NSURLConnection*)connection {
    if ([_delegate respondsToSelector:@selector(connectionContainerShouldUseCredentialStorage:)])
        return [_delegate connectionContainerShouldUseCredentialStorage:self];
    
    return YES;
}



#pragma mark -
#pragma mark Connection uploading progress delegate
- (void)connection:(NSURLConnection *)connection didSendBodyData:(NSInteger)bytesWritten totalBytesWritten:(NSInteger)totalBytesWritten totalBytesExpectedToWrite:(NSInteger)totalBytesExpectedToWrite {
    _bytesExpectedToWrite = totalBytesExpectedToWrite;
    _bytesWritten = totalBytesWritten;
    
    if ([_uploadingProgressDelegate respondsToSelector:@selector(connectionContainer:didSendBodyData:totalBytesWritten:totalBytesExpectedToWrite:)])
        [_uploadingProgressDelegate connectionContainer:self didSendBodyData:bytesWritten totalBytesWritten:totalBytesWritten totalBytesExpectedToWrite:totalBytesExpectedToWrite];
}



#pragma mark -
#pragma mark Connection downloading progress delegate
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [_responseData appendData:data];
    _bytesRead += [data length];
    
    if ([_downlodingProgressDelegate respondsToSelector:@selector(connectionContainer:didReceiveData:)])
        [_downlodingProgressDelegate connectionContainer:self didReceiveData:data];
}




#pragma mark -
#pragma mark Properties
- (NSString *)responseString {
    if (!_responseString) {
        NSStringEncoding encoding = _responseStringEncoding ? _responseStringEncoding : NSUTF8StringEncoding;
        _responseString = [[NSString alloc] initWithData:_responseData
                                                encoding:encoding];
    }
    
    return _responseString;
}



- (NSArray *)responseCookies {
    if (!_responseCookies && [_urlResponse isKindOfClass:[NSHTTPURLResponse class]]) {
        NSString *urlString = [NSString stringWithFormat:@"%@://%@", [_request.URL scheme], [_request.URL host]];
        NSURL *url = [NSURL URLWithString:urlString];
        
        _responseCookies = [[NSHTTPCookie cookiesWithResponseHeaderFields:[(NSHTTPURLResponse *)_urlResponse allHeaderFields] forURL:url] retain];
    }
    
    return _responseCookies;
}

@end