/*
 * This file is part of the SDWebImage package.
 * (c) Olivier Poitrey <rs@dailymotion.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

#import "UIImageView+WebCache.h"
#import "SDImageCache.h"
#import <objc/runtime.h>


@interface UIImageView ()

@property (nonatomic, readwrite, retain) NSURL *loadingImageURL;

@end



@implementation UIImageView (WebCache)

- (void)setImageWithURL:(NSURL *)url
{
    [self setImageWithURL:url placeholderImage:nil];
}

- (void)setImageWithURL:(NSURL *)url placeholderImage:(UIImage *)placeholder
{
    [self setImageWithURL:url placeholderImage:placeholder options:0];
}

- (void)setImageWithURL:(NSURL *)url placeholderImage:(UIImage *)placeholder options:(SDWebImageOptions)options
{
    [self setImageWithURL:url placeholderImage:placeholder options:options success:nil failure:nil];
}


- (void)setImageWithURL:(NSURL *)url success:(void (^)(UIImage *image))success failure:(void (^)(NSError *error))failure;
{
    [self setImageWithURL:url placeholderImage:nil success:success failure:failure];
}

- (void)setImageWithURL:(NSURL *)url placeholderImage:(UIImage *)placeholder success:(void (^)(UIImage *image))success failure:(void (^)(NSError *error))failure;
{
    [self setImageWithURL:url placeholderImage:placeholder options:0 success:success failure:failure];
}

- (void)setImageWithURL:(NSURL *)url placeholderImage:(UIImage *)placeholder options:(SDWebImageOptions)options success:(void (^)(UIImage *image))success failure:(void (^)(NSError *error))failure;
{
//    [[SDImageCache sharedImageCache] clearDisk];
    
    // Remove in progress downloader from queue
    [self cancelCurrentImageLoad];
    self.image = placeholder;
    
    if (url) {
        self.loadingImageURL = url;
        [self.activityIndicator startAnimating];
        
        // Try to load processed image from cache
        if (self.postProcessingOperation && self.postProcessingCacheKeyPrefix) {
            [self loadProcessedImageWithURL:url options:options success:success failure:failure];
        }
        // Load image by URL
        else {
            [self loadImageWithURL:url options:options success:success failure:failure];
        }
    }
}



- (NSString *)processedCacheKeyForURL:(NSURL *)url {
    if (!url) return nil;
    return [self.postProcessingCacheKeyPrefix stringByAppendingString:[url absoluteString]];
}



- (void)loadProcessedImageWithURL:(NSURL *)url options:(SDWebImageOptions)options success:(void (^)(UIImage *image))success failure:(void (^)(NSError *error))failure {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_LOW, 0), ^{
        UIImage *processedImage = nil;
        NSString *processedCacheKey = [self processedCacheKeyForURL:url];
        if (processedCacheKey) processedImage = [[SDImageCache sharedImageCache] imageFromKey:processedCacheKey];
        dispatch_async(dispatch_get_main_queue(), ^{
            if (processedImage) {
                self.image = processedImage;
                self.loadingImageURL = nil;
                [self.activityIndicator stopAnimating];
            }
            else [self loadImageWithURL:url options:options success:success failure:failure];
        });
    });
}



- (void)loadImageWithURL:(NSURL *)url options:(SDWebImageOptions)options success:(void (^)(UIImage *image))success failure:(void (^)(NSError *error))failure {
    SDWebImageManager *manager = [SDWebImageManager sharedManager];
    if (success || failure) [manager downloadWithURL:url delegate:self options:options success:success failure:failure];
    else [manager downloadWithURL:url delegate:self options:options];
}



- (void)setLoadedImage:(UIImage *)image withURL:(NSURL *)url {
    ImageProcessingOperation postProcessingOperation = self.postProcessingOperation;
    if (postProcessingOperation) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_LOW, 0), ^{
            // Process image
            UIImage *processedImage = postProcessingOperation(image);
            
            // Cache processed image
            if (self.postProcessingCacheKeyPrefix) {
                NSString *processedCacheKey = [self processedCacheKeyForURL:url];
                if (processedCacheKey) [[SDImageCache sharedImageCache] storeImage:processedImage
                                                                            forKey:processedCacheKey];
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                self.image = processedImage;
                self.loadingImageURL = nil;
            });
        });
    }
    else {
        self.image = image;
        self.loadingImageURL = nil;
    }
}

- (void)cancelCurrentImageLoad
{
    self.loadingImageURL = nil;
    [[SDWebImageManager sharedManager] cancelForDelegate:self];
    [self.activityIndicator stopAnimating];
}

- (void)webImageManager:(SDWebImageManager *)imageManager didProgressWithPartialImage:(UIImage *)image forURL:(NSURL *)url
{
    [self setLoadedImage:image withURL:self.loadingImageURL];
}

- (void)webImageManager:(SDWebImageManager *)imageManager didFinishWithImage:(UIImage *)image
{
    [self setLoadedImage:image withURL:self.loadingImageURL];
    [self.activityIndicator stopAnimating];
}

- (void)webImageManager:(SDWebImageManager *)imageManager didFailWithError:(NSError *)error {
    if (self.errorImage) self.image = self.errorImage;
    [self.activityIndicator stopAnimating];
}



- (NSURL *)loadingImageURL {
    return objc_getAssociatedObject(self, "loadingImageURL");
}
- (void)setLoadingImageURL:(NSURL *)loadingImageURL {
    objc_setAssociatedObject(self, "loadingImageURL", loadingImageURL, OBJC_ASSOCIATION_RETAIN);
}



- (ImageProcessingOperation)postProcessingOperation {
    return objc_getAssociatedObject(self, "postProcessingOperation");
}
- (void)setPostProcessingOperation:(ImageProcessingOperation)postProcessingOperation {
    objc_setAssociatedObject(self, "postProcessingOperation", postProcessingOperation, OBJC_ASSOCIATION_COPY);
}



- (NSString *)postProcessingCacheKeyPrefix {
    return objc_getAssociatedObject(self, "postProcessingCacheKeyPrefix");
}
- (void)setPostProcessingCacheKeyPrefix:(NSString *)postProcessingCacheKeyPrefix {
    objc_setAssociatedObject(self, "postProcessingCacheKeyPrefix", postProcessingCacheKeyPrefix, OBJC_ASSOCIATION_RETAIN);
}



- (UIActivityIndicatorView *)activityIndicator {
    if (self.doNotShowActivityIndicator) return nil;
    UIActivityIndicatorView *result = objc_getAssociatedObject(self, "activityIndicator");
    
    if (!result) {
        result = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
        result.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin;
        result.hidesWhenStopped = YES;
        
        CGRect frame = result.frame;
        frame.origin.x = (self.frame.size.width - frame.size.width) / 2;
        frame.origin.y = (self.frame.size.height - frame.size.height) / 2;
        result.frame = CGRectIntegral(frame);
        
        [self addSubview:result];
        self.activityIndicator = result;
        [result release];
    }
    
    return result;
}
- (void)setActivityIndicator:(UIActivityIndicatorView *)activityIndicator {
    objc_setAssociatedObject(self, "activityIndicator", activityIndicator, OBJC_ASSOCIATION_RETAIN);
}



- (BOOL)doNotShowActivityIndicator {
    return [objc_getAssociatedObject(self, "doNotShowActivityIndicator") boolValue];
}
- (void)setDoNotShowActivityIndicator:(BOOL)doNotShowActivityIndicator {
    objc_setAssociatedObject(self, "doNotShowActivityIndicator", @(doNotShowActivityIndicator), OBJC_ASSOCIATION_ASSIGN);
}



- (UIImage *)errorImage {
    return objc_getAssociatedObject(self, "errorImage");
}
- (void)setErrorImage:(UIImage *)errorImage {
    objc_setAssociatedObject(self, "errorImage", errorImage, OBJC_ASSOCIATION_RETAIN);
}

@end
